package com.nominanuda.lang;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

public class MultiInterfaceFacade implements InvocationHandler {
	private List<MethodWrapper> methods = new LinkedList<MethodWrapper>();

	public MultiInterfaceFacade(Object... objs) {
		for(Object o : objs) {
			for(Class<?> iface : o.getClass().getInterfaces()) {
				for(Method m : iface.getMethods()) {
					MethodWrapper w = new MethodWrapper(m ,o);
					if(! methods.contains(w)) {
						methods.add(w);
					}
				}
			}
		}
		
	}

	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {
		for(MethodWrapper w : methods) {
			if(w.matches(method)) {
				return w.invoke(args);
			}
		}
		throw new IllegalStateException("method not found "+method.getName());
	}

	public static Object wrap(Object... objs) {
		InvocationHandler handler = null;
		Set<Class<?>> interfaces = new LinkedHashSet<Class<?>>();
		for(Object o : objs) {
			for(Class<?> iface : o.getClass().getInterfaces()) {
				interfaces.add(iface);
			}
		}
		Class<?>[] ifaces = new LinkedList<Class<?>>(interfaces).toArray(new Class[interfaces.size()]);
		Object proxy = Proxy.newProxyInstance(
			objs[0].getClass().getClassLoader(), ifaces, handler);
		return proxy;
	}
	private class MethodWrapper {
		ReflectionHelper rh = new ReflectionHelper();
		private final Method m;
		private final Object o;

		public MethodWrapper(Method m, Object o) {
			this.m = m;
			this.o = o;
		}

		public Object invoke(Object[] args) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException {
			return m.invoke(o, args);
		}

		public boolean matches(Method method) {
			
			return false;
		}

		@Override
		public boolean equals(Object arg0) {
			// TODO Auto-generated method stub
			return super.equals(arg0);
		}

		public int hashCode() {
			return m.hashCode();
		}

		public String toString() {
			return m.toString();
		}
	}
}
